-- |
-- Module      : Data.ASN1.Object
-- License     : BSD-style
-- Maintainer  : Vincent Hanquez <vincent@snarc.org>
-- Stability   : experimental
-- Portability : unknown
--
module Data.ASN1.Object {-# DEPRECATED "Use Data.ASN1.Types instead" #-}
    ( ASN1Object(..)
    ) where

import Data.ASN1.Types
